# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlinclient.common import utils


def do_command_foo(sc, args):
    """Pydoc for command foo."""
    return


@utils.arg('-F', '--flag', metavar='<FLAG>', help='Flag desc.')
@utils.arg('arg1', metavar='<ARG1>', help='Arg1 desc')
def do_command_bar(sc, args):
    """This is the command doc."""
    return
