# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlinclient.tests.functional import base


class EventTest(base.OpenStackClientTestBase):
    """Test for events"""

    def test_event_list(self):
        result = self.openstack('cluster event list')
        event_list = self.parser.listing(result)
        self.assertTableStruct(event_list, ['id', 'generated_at', 'obj_type',
                                            'obj_id', 'obj_name', 'action',
                                            'status', 'level', 'cluster_id',
                                            'meta_data'])
